<?php

require_once( 'Mail/mimeDecode.php' );

class Mail_Parser
{
    var $_parse_html = false;
    var $_parse_mime = false;
    var $_htmlparser = "/usr/bin/w3m -T text/html -dump";
    var $_htmltmpdir = "/tmp";

    var $_file;
    var $_content;
    
    var $_from;
    var $_subject;
    var $_priority;
    var $_transferencoding;
    var $_body;
    var $_parts = array ( );
    var $_ctype = array ( );

    function Mail_Parser( $options = array() ) {
	$this->_parse_mime = $options['parse_mime'];
	$this->_parse_html = $options['parse_html'];
	$this->_htmlparser = $options['htmlparser'];
	$this->_htmltmpdir = $options['htmltmpdir'];
    }
    
    function setInputString ( $content ) {
        $this->_content = $content;
    }
    
    function setInputFile( $file ) {
        $this->_file = $file;
        $this->_content = file_get_contents( $this->_file );
    }

    function parse() {
        $decoder = new Mail_mimeDecode( $this->_content );
		$params['include_bodies'] = true;
        $params['decode_bodies'] = true;
        $params['decode_headers'] = true;
        $structure = $decoder->decode( $params );
      	$this->parseStructure( $structure );
      	unset( $this->_content );
    }
    
    function from() {
	return $this->_from;
    }

    function subject() {
	return $this->_subject;
    }

    function priority() {
	return $this->_priority;
    }

    function body() {
	return $this->_body;
    }

    function parts() {
	return $this->_parts;
    }
    
    function parseStructure( $structure ) {
	$this->setFrom( $structure->headers['from'] );
	$this->setSubject( $structure->headers['subject'] );
	$this->setContentType( $structure->ctype_primary, $structure->ctype_secondary );
	if ( isset( $structure->headers['x-priority'] ) ) {
	    $this->setPriority( $structure->headers['x-priority'] );
	}
	if ( isset( $structure->headers['content-transfer-encoding'] ) ) {
	    $this->setTransferEncoding( $structure->headers['content-transfer-encoding'] );
	}
	if ( isset( $structure->body ) ) {
	    $this->setBody( $structure->body );
	}
	if ( $this->_parse_mime && isset( $structure->parts ) ) {
	    $this->setParts( $structure->parts );
	}
    }

    function setFrom( $from ) {
	$this->_from = quoted_printable_decode( $from );
    }

    function setSubject( $subject ) {
	$this->_subject = quoted_printable_decode( $subject );
    }

    function setPriority( $priority ) {
	$this->_priority = $priority;
    }

    function setTransferEncoding( $transferencoding ) {
	$this->_transferencoding = $transferencoding;
    }
    
    function setContentType( $primary, $secondary ) {
        $this->_ctype['primary'] = $primary;
	$this->_ctype['secondary'] = $secondary;
    }

    function setBody( $body ) {
        if ( 0 == strlen( $body ) || 0 != strlen( $this->_body ) ) {
	    return;
	}
	if ( 'text' == $this->_ctype['primary'] &&
	     'plain' == $this->_ctype['secondary'] ) {
	    switch ( $this->_transferencoding ) {
	        case 'base64':
	        case '8bit':
	        case 'quoted-printable':
		    $this->_body = quoted_printable_decode( $body );
		    break;
		default:
		    $this->_body = $body;
		    break;
	    }
	} elseif ( $this->_parse_html &&
	           'text' == $this->_ctype['primary'] &&
	           'html' == $this->_ctype['secondary'] ) {
            $file = $this->_htmltmpdir . "/" . md5 ( $body );
            file_put_contents( $file, $body );
            $this->_body = shell_exec("cat " . $file . " | " . $this->_htmlparser);
            unlink($file);
	}
    }
    
    function setParts( &$parts ) {
	$i = 0;
	if ( 'multipart' == $parts[$i]->ctype_primary ) {
	    $this->setParts( $parts[$i]->parts );
	    $i++;
	}
        if ( 'text' == $parts[$i]->ctype_primary &&
	    in_array( $parts[$i]->ctype_secondary, array( 'plain', 'html' ) ) ) {
	    $this->setContentType( $parts[$i]->ctype_primary, $parts[$i]->ctype_secondary );
	    $this->setTransferEncoding( $parts[$i]->headers['content-transfer-encoding'] );
	    $this->setBody( $parts[$i]->body );
	    $i++;
	}
        if ( isset( $parts[$i] ) &&
	    'text' == $parts[$i]->ctype_primary &&
	    'html' == $parts[$i]->ctype_secondary ) {
	    $i++;
	}
	for ( $i; $i < count( $parts ); $i++ ) {
	    $this->addPart( $parts[$i] );
	}
    }
    
    function addPart( &$part ) {
	$p['ctype'] = $part->ctype_primary . "/" . $part->ctype_secondary;
	if ( isset( $part->ctype_parameters['name'] ) ) {
	    $p['name'] = $part->ctype_parameters['name'];
	}
	$p['body'] = $part->body;
	$this->_parts[] = $p;
    }
}

?>
